package lancs.midp.aspects.exceptionblocks;

import java.io.IOException;
import java.io.InputStream;

import lancs.midp.mobilephoto.lib.exceptions.ImagePathNotValidException;
import lancs.midp.mobilephoto.lib.exceptions.InvalidArrayFormatException;
import lancs.midp.mobilephoto.lib.exceptions.InvalidImageDataException;
import lancs.midp.mobilephoto.lib.exceptions.InvalidImageFormatException;
import ubc.midp.mobilephoto.core.ui.datamodel.MediaData;
import ubc.midp.mobilephoto.core.util.MediaUtil;
import lancs.midp.aspects.exceptionblocks.utilAspectEH.annotations.*;

public aspect UtilAspectEH {
	
	//Method ImageUtil.readImageAsByteArray 1- block - Scenario 3
	pointcut readMediaAsByteArray(String mediaFile): 
		 (call(public void Class.getResourceAsStream(String))&&(args(mediaFile)))&& withincode(@ReadImageAsByteArrayEvent * *(..)) ;
	
	after(String mediaFile) throwing(Exception e) throws  ImagePathNotValidException: readMediaAsByteArray(mediaFile){
		throw new ImagePathNotValidException(
				"Path not valid for this image:"+mediaFile);
	}
	
	//Method ImageUtil.readImageAsByteArray 2- block - Scenario 3
	pointcut readInternalMediaAsByteArray(String mediaFile): 
		 call(@ReadInternalImageAsByteArrayEvent * *(..))&& (withincode(@ReadImageAsByteArrayEvent * *(String))&&(args(mediaFile)));
	
	declare soft: IOException: call(@ReadInternalImageAsByteArrayEvent * *(..))&& (withincode(@ReadImageAsByteArrayEvent * *(String)));
	
	after(String mediaFile) throwing(Exception e) throws  InvalidImageFormatException, ImagePathNotValidException: readInternalMediaAsByteArray(mediaFile){
		if (e instanceof IOException){
			throw new InvalidImageFormatException(
					"The file "+mediaFile+" does not have PNG format");
		}else if (e instanceof NullPointerException){
			throw new ImagePathNotValidException(
					"Path not valid for this image:"+mediaFile);
		}
	}
	
	//Method public ImageData ImageUtil.getImageInfoFromBytes(byte[] bytes) 1- block - Scenario 1
	pointcut getMediaInfoFromBytes(): 
		 execution(@GetImageInfoFromBytesEvent * *(..));
	
	after() throwing(Exception e) throws  InvalidArrayFormatException: getMediaInfoFromBytes(){
		throw new InvalidArrayFormatException();
	}
	
	//Method public byte[] ImageUtil.getBytesFromImageInfo(ImageData ii) 1- block - Scenario 1
	pointcut getBytesFromMediaInfo(): 
		 execution(@GetBytesFromImageInfoEvent * *(..));
	
	after() throwing(Exception e) throws  InvalidImageDataException: getBytesFromMediaInfo(){
		throw new InvalidImageDataException("The provided data are not valid");
	}
}