

package lancs.midp.aspects.exceptionblocks;

import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotOpenException;

import lancs.midp.mobilephoto.lib.exceptions.InvalidImageDataException;
import lancs.midp.mobilephoto.lib.exceptions.PersistenceMechanismException;
import lancs.midp.mobilephoto.lib.exceptions.UnavailablePhotoAlbumException;
import ubc.midp.mobilephoto.core.ui.datamodel.AlbumData;
import ubc.midp.mobilephoto.core.ui.datamodel.MediaAccessor;
import ubc.midp.mobilephoto.core.ui.datamodel.MediaData;
import lancs.midp.aspects.exceptionblocks.dataModelAspectEH.annotations.*;

public aspect DataModelAspectEH {
	
	//Method public void ImageMediaAccessor.addImageData(String, String, String) 1- block - Scenario 1
	pointcut addMediaData(): 
		 execution(@AddImageDataEvent * *(..));
	
	declare soft: RecordStoreException : addMediaData();
	
	after()throwing(RecordStoreException e) throws  PersistenceMechanismException: addMediaData(){
		throw new  PersistenceMechanismException();
	}
	
	//public String[] ImageMediaAccessor.loadImageDataFromRMS(String) 1- block - Scenario 3
	pointcut loadMediaDataFromRMS(): 
		 execution(@LoadImageDataFromRMSEvent * *(..));
	
	declare soft: RecordStoreException : loadMediaDataFromRMS();
	
	after()throwing(RecordStoreException e) throws  PersistenceMechanismException: loadMediaDataFromRMS(){
		throw new PersistenceMechanismException(e);
	}

	//public boolean ImageMediaAccessor.updateImageInfo(ImageData oldData, ImageData newData) block 1 - Scenario 4
	pointcut updateMediaInfo(): 
		 (call(public RecordStore RecordStore.openRecordStore(String, boolean)) || call(public void RecordStore.setRecord(..)))&& (withincode(@UpdateImageInfoEvent * *(..)));
	
	declare soft: RecordStoreException : execution(@UpdateImageInfoEvent * *(..));
	//$$$$$$$$$$$$$$$$$$$$$Check why this advice is not operating$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	//after()throwing(Exception e) throws  PersistenceMechanismException: updateImageInfo(){
	//	throw new PersistenceMechanismException(e);
	//}
	
	//public boolean ImageMediaAccessor.updateImageInfo(ImageData oldData, ImageData newData) block 2 - Scenario 6
	pointcut updateMediaInfoAround(): 
		 call(public void RecordStore.closeRecordStore(..))&& (withincode(@UpdateImageInfoEvent * *(..)));
	
	//$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$Verify why not $$$$$$$$$$$$$$$$$$$$$$$$
	//void around() throws PersistenceMechanismException: updateImageInfoAround(){
	void around(): updateMediaInfoAround(){
		try{
			proceed();
		} catch (RecordStoreNotOpenException e) {
			//No problem if the RecordStore is not Open
		} //catch (RecordStoreException e) {
		//	throw new PersistenceMechanismException(e);
		//}
	}
	
	//public byte[] ImageMediaAccessor.loadImageBytesFromRMS(String, String,int) 1- block - Scenario 3
	pointcut loadMediaBytesFromRMS(): 
		 execution(@LoadImageBytesFromRMSEvent * *(..));
	
	declare soft: RecordStoreException : loadMediaBytesFromRMS();
	
	after()throwing(RecordStoreException e) throws  PersistenceMechanismException: loadMediaBytesFromRMS(){
		throw new PersistenceMechanismException(e);
	}
	
	//public boolean ImageMediaAccessor.deleteSingleImageFromRMS(String, String) 1- block - Scenario 3
	pointcut deleteSingleMediaFromRMS(): 
		 execution(@DeleteSingleImageFromRMSEvent * *(..));
	
	declare soft: RecordStoreException : deleteSingleMediaFromRMS();
	
	after()throwing(RecordStoreException e) throws  PersistenceMechanismException: deleteSingleMediaFromRMS(){
		throw new PersistenceMechanismException(e);
	}
	
	//public void ImageMediaAccessor.createNewPhotoAlbum(String) block 1 - Scenario 4
	pointcut createNewAlbum(): 
		 (call(public RecordStore RecordStore.openRecordStore(String, boolean)) || call(public void RecordStore.closeRecordStore(..)))&& (withincode(@CreateNewPhotoAlbumEvent * *(..)));

	declare soft: RecordStoreException : createNewAlbum();
	//$$$$$$$$$$$$$$$$$$$$$Check why this advice is not operating$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	//after()throwing(Exception e) throws  PersistenceMechanismException: createNewPhotoAlbum(){
	//	throw new PersistenceMechanismException(e);
	//}
	
	//public void ImageMediaAccessor.deletePhotoAlbum(String labelName) 1- block - Scenario 3
	pointcut deleteAlbum(): 
		 execution(@DeletePhotoAlbumEvent * *(..));
	
	declare soft: RecordStoreException : deleteAlbum();
	
	after()throwing(RecordStoreException e) throws PersistenceMechanismException: deleteAlbum(){
		throw new PersistenceMechanismException(e);
	}
	
	//public String[] AlbumData.getAlbumNames() block 1 - Scenario 5
	pointcut getAlbumNames(): 
// FIXME rdyer - this breaks ajc, but works with our metrics tool!
		 call(@LoadAlbumsEvent * *(..))&& (withincode(@GetAlbumNamesEvent * *(..)));
//		 call(public void MediaAccessor.loadAlbums())&& (withincode(@GetAlbumNamesEvent * *(..)));
	
	declare soft:  InvalidImageDataException : getAlbumNames();
	declare soft:  PersistenceMechanismException : getAlbumNames();
	
	void around(): getAlbumNames(){
		try{
			proceed();
		} catch (InvalidImageDataException e) {
			e.printStackTrace();
		} catch (PersistenceMechanismException e) {
			e.printStackTrace();
		}
	}
	
	//public String[] AlbumData.getMedias(String recordName) block 1 - Scenario 3
	pointcut getMedias(): 
		 execution(@GetImagesEvent * *(..));
	
	declare soft:   PersistenceMechanismException : getMedias();
	declare soft:   InvalidImageDataException : getMedias();
	
	after()throwing(Exception e) throws  UnavailablePhotoAlbumException: getMedias(){
		if (e instanceof PersistenceMechanismException)
			throw new UnavailablePhotoAlbumException(e);
		else if (e instanceof InvalidImageDataException)
			throw new UnavailablePhotoAlbumException(e);
	}
	
	//public void AlbumData.resetImageData() block 1 - Scenario 1
	pointcut resetMediaData(): 
		execution(@ResetImageDataEvent * *(..));
	
	declare soft:InvalidImageDataException: resetMediaData();
	
	void around(): resetMediaData() {
		try{
			proceed();
		} catch (InvalidImageDataException e) {
			e.printStackTrace();
		}
	}
	
 }
